;Structures
Structure PDF_OutLineStructure 
  Level.b 
  Y.f 
  wPage.w 
  Text$
  lParent.l 
  lLast.l
  lFirst.l 
  lPrev.l
  lNext.l 
EndStructure 

;Global variables
Global pdfOutlineRoot.w
Global pdfDisplayPreferences$
Global pdfJavaScript$
Global pdfJSNum.w

;Internal function declarations
Declare ipf_PutBookMarks()
Declare ipf_PutCatalogBookMark()
Declare ipf_EndDocBookMark()
Declare ipf_PutCatalogDisplay()
Declare ipf_PutJavaScript()
Declare ipf_PutCatalogJavaScript()

;Constructor
ProcedureDLL pdfMisc_Init()
  NewList OutLines.PDF_OutLineStructure()   ;Bookmark variable
  NewList Lru.b()                           ;Bookmark variable
  ipf_SetProcPutResources(@ipf_PutBookMarks())
  ipf_SetProcPutResources(@ipf_PutJavaScript())
  ipf_SetProcPutCatalog(@ipf_PutCatalogBookMark())
  ipf_SetProcPutCatalog(@ipf_PutCatalogDisplay())
  ipf_SetProcPutCatalog(@ipf_PutCatalogJavaScript())
  ipf_SetProcEndDoc(@ipf_EndDocBookMark())  
EndProcedure

Procedure pdf_BookMarkBasic(Text$, level.b, y.f) 
  If y=-1 
    y = pdf_GetY() 
  EndIf 
  AddElement(OutLines()) 
  OutLines()\Text$ = Text$
  OutLines()\Y = y 
  OutLines()\Level = level 
  OutLines()\wPage = pdf_GetPageNo()   
  OutLines()\lParent = -1   
  OutLines()\lNext = -1  
  OutLines()\lPrev = -1  
  OutLines()\lLast = -1   
  OutLines()\lFirst = -1  
EndProcedure 

ProcedureDLL pdf_BookMark(Text$) ;Add bookmark
  pdf_BookMarkBasic(Text$,0,0) 
EndProcedure 
 
ProcedureDLL pdf_BookMark2(Text$, Level.b) ;Add bookmark
  pdf_BookMarkBasic(Text$,level,0)
EndProcedure

ProcedureDLL pdf_BookMark3(Text$, Level.b, Y.f) ;Add bookmark
  pdf_BookMarkBasic(Text$,level,y)
EndProcedure

Procedure ipf_PutBookMarks() 
  Protected nb.w
  Protected ln.w
  Protected level.b
  Protected i.l
  Protected lParent.l
  Protected lPrev.l
  Protected h.f
  Protected k.f
  
  nb = CountList(OutLines()) 
  If nb = 0 
    ProcedureReturn 0 
  EndIf 
  level=0 
  i=0 
  ForEach OutLines() 
    If (OutLines()\Level>0) 
      SelectElement(lru(), OutLines()\Level - 1)
      lparent = lru() 
      ;Set Parent And Last Pointers 
      OutLines()\lParent = lparent 
      SelectElement(OutLines(), lParent) 
      OutLines()\lLast = i 
      SelectElement(Outlines(), i)
      If (OutLines()\Level > level) 
        ;Level increasing: set first pointer 
        SelectElement(OutLines(), lParent) 
        OutLines()\lFirst=i 
      EndIf 
      SelectElement(OutLines(),i) 
    Else 
      OutLines()\lParent = nb 
    EndIf 
    
    If (OutLines()\Level<=level And i>0)       
      ;Set prev And Next pointers
      SelectElement(lru(), OutLines()\Level)
      lprev=lru() 
      SelectElement(OutLines(),lprev) 
      OutLines()\lNext = i 
      SelectElement(OutLines(), i) 
      OutLines()\lPrev = lprev 
    EndIf  
    level = OutLines()\Level
    If CountList(lru()) <= level
      AddElement(lru())
    EndIf
    SelectElement(lru(), level)
    lru()=i
    i=i+1
  Next 
  
  ;Outline items
  ln=ipf_GetN() + 1 
  ForEach OutLines()
    ipf_NewObj() 
    ipf_Out("<</Title " + ipf_TextString(OutLines()\Text$)) 
    ipf_Out("/Parent " + Str(ln + OutLines()\lParent) + " 0 R")
    If (OutLines()\lPrev <> -1) 
      ipf_Out("/Prev " + Str(ln + OutLines()\lPrev) + " 0 R") 
    EndIf 
    If (OutLines()\lNext <> -1)
      ipf_Out("/Next " + Str(ln + OutLines()\lNext) + " 0 R") 
    EndIf     
    If (OutLines()\lFirst <> -1)
      ipf_Out("/First " + Str(ln + OutLines()\lFirst) + " 0 R") 
    EndIf     
    If (OutLines()\lLast <> -1)
      ipf_Out("/Last " + Str(ln + OutLines()\lLast) + " 0 R") 
    EndIf     
    h = ipf_GetH()
    k = ipf_GetK()
    ipf_Out("/Dest [" + Str(1+2*OutLines()\wPage) + " 0 R /XYZ 0 " + StrF((h-OutLines()\Y)*k,2) + " null]")

    ipf_Out("/Count 0>>") 
    ipf_Out("endobj") 
  Next

  ;OutLine root 
  ipf_NewObj() 
  pdfOutlineRoot = ipf_GetN()
  ipf_Out("<</Type /Outlines /First " + Str(ln) + " 0 R") 
  SelectElement(lru(),0)
  ipf_Out("/Last " + Str(ln + lru()) + " 0 R>>")
  ipf_Out("endobj") 
  ProcedureReturn 0  
EndProcedure 

Procedure ipf_PutCatalogBookMark()
  If(CountList(OutLines()) > 0) 
    ipf_Out("/Outlines " + Str(pdfOutlineRoot) + " 0 R") 
    ipf_Out("/PageMode /UseOutlines") 
  EndIf 
EndProcedure

Procedure ipf_EndDocBookMark()
  ClearList(OutLines())
  ClearList(lru())
EndProcedure

ProcedureDLL pdf_DisplayPreferences(Preferences$) ;Set viewer preferences
  pdfDisplayPreferences$ = Preferences$
EndProcedure

Procedure ipf_PutCatalogDisplay()
  If FindString(pdfDisplayPreferences$,"FullScreen",1) > 0
    ipf_Out("/PageMode /FullScreen")  
  EndIf

  If Len(pdfDisplayPreferences$) > 0
    ipf_Out("/ViewerPreferences<<")
    If FindString(pdfDisplayPreferences$,"HideMenubar",1) > 0
      ipf_Out("/HideMenubar true")  
    EndIf
    If FindString(pdfDisplayPreferences$,"HideToolbar",1) > 0
      ipf_Out("/HideToolbar true")  
    EndIf
    If FindString(pdfDisplayPreferences$,"HideWindowUI",1) > 0
      ipf_Out("/HideWindowUI true")  
    EndIf
    If FindString(pdfDisplayPreferences$,"DisplayDocTitle",1) > 0
      ipf_Out("/DisplayDocTitle true")  
    EndIf
    If FindString(pdfDisplayPreferences$,"CenterWindow",1) > 0
      ipf_Out("/CenterWindow true")  
    EndIf
    If FindString(pdfDisplayPreferences$,"FitWindow",1) > 0
      ipf_Out("/FitWindow true")  
    EndIf
    ipf_Out(">>")
  EndIf
EndProcedure

ProcedureDLL pdf_IncludeJS(Script$) ;Add JavaScript
  pdfJavaScript$ = Script$
EndProcedure

Procedure ipf_PutJavascript()
  If pdfJavaScript$<>""
    ipf_NewObj()
    pdfJSNum = ipf_GetN()
    ipf_Out("<<")
    ipf_Out("/Names [(EmbeddedJS) " + Str(ipf_GetN() + 1) + " 0 R ]")
    ipf_Out(">>")
    ipf_Out("endobj")
    ipf_NewObj()
    ipf_Out("<<")
    ipf_Out("/S /JavaScript")
    ipf_Out("/JS " + ipf_TextString(pdfJavaScript$))
    ipf_Out(">>")
    ipf_Out("endobj")    
  EndIf  
EndProcedure

Procedure ipf_PutCatalogJavaScript()
  If pdfJavaScript$ <> ""
    ipf_Out("/Names <</JavaScript " + Str(pdfJSNum) + " 0 R>>")
  EndIf
EndProcedure

Procedure pdf_MultiCellBltBasic(w.f,h.f, blt$,txt$,border.f,align$,fill.f)
  Protected blt_width.f
  Protected bak_x.f
  Protected cMargin.f
  cMargin = ipf_GetCMargin()
  ;Get bullet width including margins
  blt_width = pdf_GetStringWidth(blt$) + cMargin*2
  ;Save x
  bak_x = pdf_GetX()
  ;Output bullet
  pdf_Cell(blt_width,h,blt$,0,0,"",fill)
  ;Output text
  pdf_MultiCell(w - blt_width, h, txt$, border, align$, fill,0)
  ;Restore x
  pdf_SetX(bak_x)  
EndProcedure

ProcedureDLL pdf_MultiCellBlt(W.f, H.f, Blt$, Txt$) ;Add multicell with bullet
  pdf_MultiCellBltBasic(w, h, blt$, txt$, 0, "J", -1)
EndProcedure

ProcedureDLL pdf_MultiCellBlt2(W.f, H.f, Blt$, Txt$, Border.f) ;Add multicell with bullet
  pdf_MultiCellBltBasic(w, h, blt$, txt$, border, "J", -1)
EndProcedure

ProcedureDLL pdf_MultiCellBlt3(W.f, H.f, Blt$, Txt$, Border.f, Align$) ;Add multicell with bullet
  pdf_MultiCellBltBasic(w,h, blt$,txt$, border, align$,-1)
EndProcedure

ProcedureDLL pdf_MultiCellBlt4(W.f, H.f, Blt$, Txt$, Border.f, Align$, Fill.f) ;Add multicell with bullet
  pdf_MultiCellBltBasic(w,h, blt$,txt$, border, align$, fill)
EndProcedure

Procedure pdf_SubWriteBasic(h.f, txt$, link.l,subFontSize.b, subOffset.f)
  subFontSizeold.b = ipf_GetFontSizePt()
	pdf_SetFontSize(subFontSize)
	
	pdfK.f = ipf_GetK()
	
	subOffset = (((subFontSize - subFontSizeold) / pdfK) * 0.3) + (subOffset / pdfK)
	subX.f    = pdf_GetX()
	subY.f    = pdf_GetY()

	pdf_SetXY(subX, subY - subOffset)
	
  pdf_Write(h, txt$, link)

	subX.f    = pdf_GetX()
	subY.f    = pdf_GetY()
	pdf_SetXY(subX ,  subY + subOffset)

	pdf_SetFontSize(subFontSizeold)
EndProcedure

ProcedureDLL pdf_SubWrite(H.f, Txt$)
  pdf_SubWriteBasic(h, txt$, -1, 12, 0)
EndProcedure

ProcedureDLL pdf_SubWrite2(H.f, Txt$, Link.l)
  pdf_SubWriteBasic(h, txt$, link, 12, 0)
EndProcedure

ProcedureDLL pdf_SubWrite3(H.f, Txt$, Link.l, SubFontSize.b)
  pdf_SubWriteBasic(h, txt$, link, SubFontSize, 0)
EndProcedure

ProcedureDLL pdf_SubWrite4(H.f, Txt$, Link.l, SubFontSize.b, SubOffSet.b)
  pdf_SubWriteBasic(h, txt$, link, SubFontSize, SubOffSet)
EndProcedure
; IDE Options = PureBasic v3.94 (Windows - x86)