;The key method used is pdf_AcceptPageBreak(). It allows to accept or not an automatic page break. 
;By refusing it and altering the margin and current position, the desired column layout is achieved. 
;For the rest, not much change; two properties have been added to save the current column number 
;and the position where columns begin, and the pdf_MultiCell() call specifies a 6 centimeter width. 

Global title.s
Global col.f
Global y0.f
title="20000 Leagues Under the Seas"

Procedure Header()
  pdf_SetFont("Arial","B",15)
  w = pdf_GetStringWidth(title)+6
  pdf_SetX((210 - w)/2)
  pdf_SetDrawColor(0,80,180)
  pdf_SetFillColor(230,230,0)
  pdf_SetTextColor(220,50,50)
  pdf_SetLineWidth(1)
  pdf_Cell(w,9,title,1,1,#PDF_ALIGN_CENTER,1)
  pdf_Ln(10)
  y0 = pdf_GetY()
EndProcedure

Procedure footer()
  pdf_SetY(-15)
  pdf_SetFont("Arial","I",8)
  pdf_SetTextColor(128)
  txt.s = "Page " + Str(pdf_GetPageNo())
  pdf_Cell(0,10,txt,0,0,#PDF_ALIGN_CENTER)
EndProcedure

Procedure SetCol(cl.f)
  ;Set position at a given column
  col = cl;
  x=10+cl*65;
  pdf_SetLeftMargin(x);
  pdf_SetX(x);
EndProcedure

Procedure AcceptPageBreak()
  ;Method accepting Or not automatic page Break
  If(col<2)
    ;Go To Next column
    SetCol( col+1)
    ;Set ordinate To top
    
    pdf_SetY(y0)
    ;Keep on page
    ProcedureReturn #False
 Else
  ;Go back To first column
  SetCol(0)
  ;Page Break
  ProcedureReturn #True;
  EndIf
EndProcedure



Procedure ChapterTitle(num, label.s)
  pdf_SetFont("Arial", "",12)
  pdf_SetFillColor(200,220,255)
  pdf_Cell(0,6,"Chapter " + Str(num) + " : " + label,0,1,#PDF_ALIGN_LEFT,1)
  pdf_Ln(4)
  y0=pdf_GetY()
EndProcedure

Procedure ChapterBody(file.s)
  txt.s = ""
  OpenFile(0,file)
  l = FileSize(file)
  *MemoryBuffer = AllocateMemory(l) 

  l = ReadData(*MemoryBuffer, l)
  CloseFile(0)
  txt = PeekS(*MemoryBuffer ,l) 
  pdf_SetFont("Times","",12)
  pdf_MultiCell(60,5,txt)
  pdf_Ln()
  pdf_SetFont("","I")
  pdf_Cell(0,5,"(end of excerpt)")
  setcol(0)
  
EndProcedure

Procedure PrintChapter(num,atitle.s,file.s)
  pdf_AddPage()
  ChapterTitle(num,atitle)
  ChapterBody(file)
EndProcedure

pdf_Create()
pdf_SetProcFooter(@Footer())
pdf_SetProcHeader(@Header())
pdf_SetProcAcceptPageBreak(@AcceptPageBreak())
pdf_SetTitle(title)
pdf_SetAuthor("Jules Verne")

PrintChapter(1,"A RUNAWAY REEF","20k_c1.txt");
PrintChapter(2,"THE PROS And CONS","20k_c2.txt");
pdf_Save("Tutorial04.pdf")


; IDE Options = PureBasic v3.94 (Windows - x86)